#############
# Created By: ajb
# Maintainer: $Author: gq1 $
# Created On: 2009-10-08
# Last Modified: $Date: 2010-12-02 12:30:28 +0000 (Thu, 02 Dec 2010) $
# $Id: finder.pm 12048 2010-12-02 12:30:28Z gq1 $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/useful_modules/tags/release-31.0/lib/npg_common/run/finder.pm $

package npg_common::run::finder;

use Moose;
use MooseX::StrictConstructor;
use Carp;
use English qw{-no_match_vars};
use File::Spec::Functions qw(catfile);

with 'npg_common::roles::run::short_info',
     'npg_common::roles::run::path_info';

use Readonly; Readonly::Scalar our $VERSION => do { my ($r) = q$LastChangedRevision: 12048 $ =~ /(\d+)/mxs; $r; };

Readonly::Scalar our $FUSE_PATH => q[/fuse/mpsafs];


has '_long_store_path'   => (
                             isa            => 'Str',
                             is             => 'ro',
                             required       => 0,
                             reader         => 'long_store_path',
                             default        => $FUSE_PATH,
		            );

has 'short_store_lookup' => (
                             isa            => 'Bool',
                             is             => 'ro',
                             required       => 0,
                             default        => 1,
			    );


has 'long_store_lookup' =>  (
                             isa            => 'Bool',
                             is             => 'ro',
                             required       => 0,
                             default        => 1,
			    );


has 'archive_folder'   => (
                            isa            => 'Str',
                            is             => 'ro',
                            required       => 0,
                            lazy_build     => 1,
			   );
sub _build_archive_folder {
    my $self = shift;

    my $run_dir = q[];
    if ($self->long_store_lookup) {
        $run_dir = catfile($self->long_store_path, q[runs], $self->id_run);
        if (!(-d $run_dir)) {
            $run_dir = q[];
        }
    }

    if (!$run_dir && $self->short_store_lookup) {
        ##no critic (RequireCheckingReturnValueOfEval)
        eval { $run_dir = $self->archive_path; };
        ##use critic
    }

    if (!$run_dir) {
        croak q[Archive directory for run ] . $self->id_run . q[ not found];
    }
    return $run_dir;
}


sub found_in_long_store {

    my $self = shift;

    my $location;
    ##no critic (RequireCheckingReturnValueOfEval)
    eval {
        $location = $self->archive_folder;
    };
    ##use critic
    my $long_store = $self->long_store_path;
    if ($location && $location =~ /^$long_store/smx) {
        return 1;
    }
    return 0;
}

no Moose;
__PACKAGE__->meta->make_immutable;

1;

__END__

=head1 NAME

npg_common::run::finder

=head1 VERSION

$LastChangedRevision: 12048 $

=head1 SYNOPSIS

  my $finder = npg_common::run::finder->new(id_run => 2222);
  my $dir = $finder->archive_folder;

=head1 DESCRIPTION

Located the folder with output files for a particular run. Transparently searches different
possible locations.

=head1 SUBROUTINES/METHODS

=head2 long_store_path - read-only accessor, returns a path to long store

=head2 id_run - run identifies, should be supplied in teh constructor

=head2 short_store_lookup - a boolean flag indicating whether the temporary storage should be searched

=head2 archive_folder - accessor, returns a path to the archive

=head2 found_in_long_store - returns 1 if the run archive folder is found in a long store, 0 otherwise.
Lazy_builds the archive folder attribute if necessary.

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item Moose

=item MooseX::StrictConstructor

=item Carp

=item English -no_match_vars

=item Readonly

=item File::Spec::Functions

=item npg_common::roles::run::short_info

=item npg_common::roles::run::path_info

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

$Author: gq1 $

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2010 GRL, by Andy Brown and Marina Gourtovaia

This file is part of NPG.

NPG is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

=cut
