#!/software/bin/perl -w

#########
# Author:        gq1
# Maintainer:    $Author: gq1 $
# Created:       2009-04-20
# Last Modified: $Date: 2010-12-02 12:30:28 +0000 (Thu, 02 Dec 2010) $
# Id:            $Id: fastq_split.pl 12048 2010-12-02 12:30:28Z gq1 $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/useful_modules/tags/release-31.0/scripts/fastq_split.pl $
#

use strict;
use warnings;
use lib qw(lib /software/solexa/lib/perl5/);
use Carp;
use English qw{-no_match_vars};
use Getopt::Long;
use npg_common::Fastq_Split;

use Readonly; Readonly::Scalar our $VERSION => do { my ($r) = q$LastChangedRevision: 12048 $ =~ /(\d+)/mx; $r; };

main();
0;

sub main {
  my $opts = initialise();

  my $bwa_cmd = '/software/solexa/bin/bwa';
  my $bwa_options = '-t 2';
  my $samtools_cmd = '/software/solexa/bin/aligners/bin/samtools';

  #default human reference locations
  $opts->{ref_bwa} ||= '/nfs/repository/d0031/references/Human/NCBI36/all/bwa/Homo_sapiens.NCBI36.48.dna.all.fa';
  $opts->{ref_fa}  ||= '/nfs/repository/d0031/references/Human/NCBI36/all/fasta/Homo_sapiens.NCBI36.48.dna.all.fa';

  if(!$opts ||  $opts->{help} ||  scalar keys %{$opts} <3 ){
    print q[Usage:
    fastq_split

    Options:
      -fastq1     filename1
      -fastq2     filename2
      -fastqt     filename_t
      -srf        srf_filename
      -srf_filter
      -help
      -output_dir output_dir
      -ref_bwa    ref_bwa_prefix
      -ref_fa     ref_fa
      -ref_name   reference_name
      -fork_alignment
      -id_run     run_id
      -position   position
      -stats_only
      -no_pileup
      
    At least one fastq file needs to be provided by fastq1 and fastq2.
    output_dir must be given.
    Optional fastqt for tags fastq file to be split.
    One srf file can be given to be split, but it will need more memory.
    If srf_filter option given, then two srf_filter files will be generated:
    srf_human.filter and srf_nonhuman.filter. 
    These two files can be used for srf_filter command to split the srf file later
    in this case srf file is not necessary to be given.
    Optional reference location in BWA format, default human reference.
    Optional Fasta format with samtools fai, default human reference.
    Optional ref_name, default human.
    Optional id_run and position.
]
    or croak qq{Error printing help: $EVAL_ERROR};
    exit 2;
  }

  print {*STDERR} qq{Processing spliting fastq files...\n};
  my ($sec,$min,$hour,$day,$month,$yr19,@rest) =   localtime(time);
  printf {*STDERR} qq{Time:\t%02d:%02d:%02d\n}, $hour, $min, $sec;

  my $fastq_split = npg_common::Fastq_Split->new({
      bwa_cmd         => $bwa_cmd,
      bwa_options     => $bwa_options,
      samtools_cmd    => $samtools_cmd,
  });
  $fastq_split->file_split($opts);

  ($sec,$min,$hour,$day,$month,$yr19,@rest) =   localtime(time);
  printf {*STDERR} qq{Time:\t%02d:%02d:%02d\n}, $hour, $min, $sec;
  print {*STDERR} "finish spliting fastq files\n";

}

# ----------------------------------------------------------------------
sub initialise {

    my %options = ();

    my $rc = GetOptions(\%options,
                        'help',
                        'fastq1:s',
                        'fastq2:s',
                        'fastqt:s',
                        'srf:s',
                        'id_run:i',
                        'position:i',
                        'srf_filter',
                        'output_dir:s',
                        'ref_bwa:s',
                        'ref_fa:s',
                        'ref_name:s',
                        'fork_alignment',
                        'stats_only',
                        'no_pileup', 
                        );
    return \%options;

}
