use strict;
use warnings;
use Carp;
use English qw{-no_match_vars};
use Test::More tests => 7;
use Test::Deep;
use Test::Exception;

BEGIN {
  use_ok('npg_common::config::roles::db_connect');
}

{
  package t::connection;
  use Moose;

  sub connection {
    my ($self, @info) = @_;
    return @info;
  }
  sub storage {return 0;}
}

{
  package t::magic::connection;
  use Moose;
  extends 't::connection';
  with 'npg_common::config::roles::db_connect';
}

{
  my $mc = t::connection->new();
  is (join(q[:], $mc->connection((2, 3))), q[2:3], 'info in and out for a base object');
  is ($mc->connection(), 0, 'undef in and out for a base object');
}

my $dsn      = q[DBI:mysql:database=testname;host=testhost;port=9999];
my $user     =  q[testuser];
my $password =  q[testpass];

{
  local $ENV{dev} = q[test];
  my $mc = t::magic::connection->new();
  is (join(q[:], $mc->connection((2, 3))), q[2:3], 'info in and out for a magic object');
  is (join(q[ ],  $mc->connection()), join(q[ ], $dsn, $user, $password), 'info out with nothing in for a magic object, dev=test');
}

{
  local $ENV{dev} = q[test];
  my $mc = t::magic::connection->new();
  my @expected = ({dsn=>$dsn, user=>$user, password=>$password, dodo=>1,});
  cmp_deeply ($mc->connection(({dsn => q[], dodo=>1,})), @expected, 'magic object: info out with array with a no-dsn hash in, dev=test');
}

{
  local $ENV{dev} = q[test];
  my $mc = t::magic::connection->new();
  my @expected = ({dsn=>q[dada], dodo=>1,});
  cmp_deeply ($mc->connection(({dsn => q[dada], dodo=>1,})), @expected, 'magic object: info out with array with a hash in, dev=test');
}

1;