use strict;
use warnings;
use English;
use Test::More tests => 10;

use_ok( 'npg_common::diagram::visio_histo_google');

{
  my $dia = npg_common::diagram::visio_histo_google->new();

  isa_ok($dia, 'npg_common::diagram::visio_histo_google');

  is($dia->get_diagram_string(), 'http://chart.apis.google.com/chart?chbh=5,1,1&chco=4D89F9&chd=t:20,30,40,23&chds=0,40&chs=300x250&cht=bvg&chtt=batch+X,run+Y,+lane+Z&chxr=0,1,4|1,0,40,10&chxt=x,y', 'default diagram string');
}


{
  my $dia = npg_common::diagram::visio_histo_google->new();
  my $data = [2,3,4];
  $dia->set_data($data);
  is($dia->get_diagram_string(), 'http://chart.apis.google.com/chart?chbh=5,1,1&chco=4D89F9&chd=t:2,3,4&chds=0,40&chs=300x250&cht=bvg&chtt=batch+X,run+Y,+lane+Z&chxr=0,1,4|1,0,40,10&chxt=x,y', 'set_data');
}

{
  my $dia = npg_common::diagram::visio_histo_google->new();
  $dia->set_axisY_min_max(2,6);
  is($dia->get_diagram_string(), q[http://chart.apis.google.com/chart?chbh=5,1,1&chco=4D89F9&chd=t:20,30,40,23&chds=2,6&chs=300x250&cht=bvg&chtt=batch+X,run+Y,+lane+Z&chxr=0,1,4|1,0,40,10&chxt=x,y], 'set_axisY_min_max');
}

{
  my $dia = npg_common::diagram::visio_histo_google->new();
  $dia->set_chart_title(q[my chart]);
  is($dia->get_diagram_string(), 'http://chart.apis.google.com/chart?chbh=5,1,1&chco=4D89F9&chd=t:20,30,40,23&chds=0,40&chs=300x250&cht=bvg&chtt=my+chart&chxr=0,1,4|1,0,40,10&chxt=x,y', 'set_title');
}


{
  my $dia = npg_common::diagram::visio_histo_google->new();
  $dia->set_chart_labels(2,5,1,4,7,1);
  is($dia->get_diagram_string(), 'http://chart.apis.google.com/chart?chbh=5,1,1&chco=4D89F9&chd=t:20,30,40,23&chds=0,40&chs=300x250&cht=bvg&chtt=batch+X,run+Y,+lane+Z&chxr=0,2,5,1|1,4,7,1&chxt=x,y', 'set labels');
}


{
  my $dia = npg_common::diagram::visio_histo_google->new();
  $dia->set_chart_size(900,200);
  is($dia->get_diagram_string(), 'http://chart.apis.google.com/chart?chbh=5,1,1&chco=4D89F9&chd=t:20,30,40,23&chds=0,40&chs=900x200&cht=bvg&chtt=batch+X,run+Y,+lane+Z&chxr=0,1,4|1,0,40,10&chxt=x,y', 'set size');
}

{
  my $dia = npg_common::diagram::visio_histo_google->new();
  $dia->set_chart_size(900,200);
  $dia->set_bar_size(7, 2);
  is($dia->get_diagram_string(), 'http://chart.apis.google.com/chart?chbh=7,1,2&chco=4D89F9&chd=t:20,30,40,23&chds=0,40&chs=900x200&cht=bvg&chtt=batch+X,run+Y,+lane+Z&chxr=0,1,4|1,0,40,10&chxt=x,y', 'set bar size');
  $dia->set_bar_size(8);
  is($dia->get_diagram_string(), 'http://chart.apis.google.com/chart?chbh=8,1,0&chco=4D89F9&chd=t:20,30,40,23&chds=0,40&chs=900x200&cht=bvg&chtt=batch+X,run+Y,+lane+Z&chxr=0,1,4|1,0,40,10&chxt=x,y', 'set bar size');
}




    