use strict;
use warnings;
use English;
use Test::More tests => 18;

my @subs = qw(get_qvals check_int);
use_ok( 'npg_common::qXvalues', @subs);

my $file = q[t/data/2549_1_2.fastqcheck];

{
  can_ok(__PACKAGE__, 'get_qvals');
}

{
  ok(npg_common::qXvalues::check_int(7), 'integer test for an integer succeeds');
  ok(npg_common::qXvalues::check_int(0), 'integer test for zero succeeds');
  ok(npg_common::qXvalues::check_int(-6), 'integer test for a negatie integer succeeds');
  is(npg_common::qXvalues::check_int(7.8), '0', 'integer test for a float fails');
  is(npg_common::qXvalues::check_int('78werwer'), '0', 'integer test for a string fails');
}


{
  eval {
    get_qvals($file);
  };
  like($EVAL_ERROR,  qr/Wrong\ number\ of\ arguments/smx, 'error on wrong number of arguments to get_qvals');

  my @thresholds = qw(20 30 35);
  eval {
    get_qvals(q[file.fastqcheck], \@thresholds);
  };
  like($EVAL_ERROR,  qr/Cannot\ open/smx, 'error on file not there in get_qvals');

  eval {
    get_qvals($file, q[sdsdsd]);
  };
  like($EVAL_ERROR,  qr/Wrong\ type\ for\ the\ second\ argument/smx, 'error on the second argument to get_qvals not an array reference');
}


{
  my @thresholds = qw(20 30 35);
  my $values = get_qvals($file, \@thresholds);
  my $string_of_values = join(';', @{$values});
  is($string_of_values, q[462472342;231502265;83021502], 'Q20, Q30 and Q35 values computed');


  @thresholds = qw(42);
  $values = get_qvals($file, \@thresholds);
  $string_of_values = join(';', @{$values});
  is($string_of_values, q[0], 'Q42 value computed');

  @thresholds = qw(38);
  $values = get_qvals(q[t/data/test.fastqcheck], \@thresholds);
  $string_of_values = join(';', @{$values});
  is($string_of_values, q[532189], 'last available  Q value computed');
}


{
  my @thresholds = qw(-2 45);
  my $values = get_qvals($file, \@thresholds);
  my $string_of_values = join(';', @{$values});
  is($string_of_values, q[0;0], 'non-available Qs returned as 0');


  @thresholds = qw(20 24.5 bb);
  $values = get_qvals($file, \@thresholds);
  $string_of_values = join(';', @{$values});
  is($string_of_values, q[462472342;0;0], 'values for illegal Qs are returned as 0');

}


{
  my @thresholds = qw(42);
  eval {
    get_qvals(q[t/data/wrong.fastqcheck], \@thresholds);
  };
  
  like($EVAL_ERROR,  qr/expected\ structure/smx, 'error on file with wrong structure');

}


{
  my @thresholds = qw(42);
  my $results = get_qvals(q[t/data/zero.fastqcheck], \@thresholds);
  is($results->[0],  0, 'zero returned for an empty fastq, one threshold as an arg');

  push @thresholds, 20;
  $results = get_qvals(q[t/data/zero.fastqcheck], \@thresholds);
  is(join(q[ ], @{$results}),  q[0 0], 'zero returned for an empty fastq, two thresholds as an arg');
}


