#########
# Author:        Marina Gourtovaia
# Maintainer:    $Author: gq1 $
# Created:       June 2010
# Last Modified: $Date: 2010-12-02 12:30:28 +0000 (Thu, 02 Dec 2010) $
# Id:            $Id: 10-roles-run-lane-map2lims.t 12048 2010-12-02 12:30:28Z gq1 $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/useful_modules/tags/release-31.0/t/10-roles-run-lane-map2lims.t $
#

package lane_maping;

use strict;
use warnings;
use Test::More tests => 9;
use Test::Exception;
use Moose::Meta::Class;

local $ENV{q[NPG_WEBSERVICE_CACHE_DIR]} = q[t/data/sequence];

use_ok('npg_common::roles::run::lane::map2lims');

{
  my $obj = Moose::Meta::Class->create_anon_class(
            roles => [qw/npg_common::roles::run::lane::map2lims
                         npg_common::roles::run::lane
                         npg_common::roles::run
                         npg_common::roles::run::lane::tag
                        /])
          ->new_object({ id_run => 4140, position=> 1,});
  my $asset = $obj->asset4expected_insert_size;
  is ($asset->id, 57224, 'asset for a normal lane');
}


{
  my $obj = Moose::Meta::Class->create_anon_class(
            roles => [qw/npg_common::roles::run::lane::map2lims
                         npg_common::roles::run::lane
                         npg_common::roles::run
                         npg_common::roles::run::lane::tag
                        /])
          ->new_object({ id_run => 4254, position=> 1,});
  my $asset = $obj->asset4expected_insert_size;
  is ($asset->type, q[MultiplexedLibraryTube], 'correct asset type for a pool lane');
  is ($asset->id, 87540, 'asset id for a pool lane');
}

{
  my $obj = Moose::Meta::Class->create_anon_class(
            roles => [qw/npg_common::roles::run::lane::map2lims
                         npg_common::roles::run::lane
                         npg_common::roles::run
                         npg_common::roles::run::lane::tag
                        /])
          ->new_object({ id_run => 4254, position=> 1, tag_index => 0,});
  my $asset = $obj->asset4expected_insert_size;
  is ($asset->type, q[MultiplexedLibraryTube], 'correct asset type for a pool lane tag index 0');
  is ($asset->id, 87540, 'asset id for a pool lane tag index 0');
}

{
  my $obj = Moose::Meta::Class->create_anon_class(
            roles => [qw/npg_common::roles::run::lane::map2lims
                         npg_common::roles::run::lane
                         npg_common::roles::run
                         npg_common::roles::run::lane::tag
                        /])
          ->new_object({ id_run => 4254, position=> 1, tag_index => 1,});
  my $asset = $obj->asset4expected_insert_size;
  is ($asset->type, q[LibraryTube], 'correct asset type for a pool lane tag index 1');
  is ($asset->id, 64422, 'asset id for a pool lane tag index 1');
}

{
  my $obj = Moose::Meta::Class->create_anon_class(
            roles => [qw/npg_common::roles::run::lane::map2lims
                         npg_common::roles::run::lane
                         npg_common::roles::run
                         npg_common::roles::run::lane::tag
                        /])
          ->new_object({ id_run => 4254, position=> 1, tag_index => 800,});
  throws_ok { $obj->asset4expected_insert_size } qr/Library tube asset for run 4254 position 1 tag index 800 not found/, 'error if invalid tag index is set';
}
