#########
# Author:        ajb
# Last Modified: $Date: 2010-12-02 12:30:28 +0000 (Thu, 02 Dec 2010) $ $Author: gq1 $
# Id:            $Id: 10-run-finder.t 12048 2010-12-02 12:30:28Z gq1 $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/useful_modules/tags/release-31.0/t/10-run-finder.t $
#

use strict;
use warnings;
use Carp;
use English qw{-no_match_vars};
use Test::More tests => 14;
use Test::Exception;

local $ENV{TEST_DIR} = q[t/data];

sub BEGIN {
  use_ok('npg_common::run::finder');
};

{
  my $finder;
  lives_ok { $finder = npg_common::run::finder->new(id_run => 22); } q{create npg_common::run::finder object ok};
  isa_ok( $finder, q[npg_common::run::finder] );
}

{
  throws_ok { npg_common::run::finder->new(id_run => 22, position => 3) } 
         qr/Found unknown attribute/, 'error when non-existing attr is set in the constructor';
}

{
  my $finder = npg_common::run::finder->new(id_run => 22);
  is ($finder->long_store_path, q[/fuse/mpsafs], 'default path to long store');
  is ($finder->short_store_lookup, 1, 'default short store lookup option');
}

{
  my $finder = npg_common::run::finder->new(_long_store_path => q[t/data/fuse/mpsafs], id_run => 1234);
  is($finder->archive_folder, q[t/data/fuse/mpsafs/runs/1234], 'existing long-term dir returned');
}

{
  my $finder = npg_common::run::finder->new(_long_store_path => q[t/data/fuse/mpsafs], id_run => 1234, long_store_lookup => 0);
  throws_ok {$finder->archive_folder} qr/Archive directory for run 1234 not found/, 'error if no directory found';
}

{
  my $finder = npg_common::run::finder->new(_long_store_path => q[t/data/fuse/mpsafs], id_run => 1235);
  throws_ok {$finder->archive_folder} qr/Archive directory for run 1235 not found/, 'error if no directory found';

  my $folder = q[t/data];
  lives_ok { $finder = npg_common::run::finder->new(
                    _long_store_path => q[t/data/fuse/mpsafs], 
                    id_run => 1235,
                    archive_folder => $folder,
           ) }
        'no error for a run not in any of the stores if the archive_folder arg is set through the constructor';
  is ($finder->archive_folder, $folder, 'archive directory is set correctly through the constructor');
}


{
  my $finder = npg_common::run::finder->new();
  throws_ok  {$finder->archive_folder} qr/Unable to obtain id_run from name/, 'error if archive_folder requested , but id_run not defined';
}

{
  my $finder = npg_common::run::finder->new(_long_store_path => q[t/data/fuse/mpsafs], id_run => 1234);
  is ($finder->found_in_long_store, 1, 'found_in_long_store returns 1 for a run in the long-term archive');
  $finder = npg_common::run::finder->new(_long_store_path => q[t/data/fuse/mpsafs], id_run => 1235);
  is ($finder->found_in_long_store, 0, 'found_in_long_store returns 0 for a run that is nowhere');
}

1;