use strict;
use warnings;
use Test::More tests => 6;
use Test::Exception;
use Perl6::Slurp;
use npg_qc::Schema;

BEGIN {
  use_ok(q{npg_common::fastqcheck});
}

{
  local $ENV{dev} = q[test];
  my $s = npg_qc::Schema->connect();
  $s->deploy;
  throws_ok { 
    npg_common::fastqcheck->new(fastqcheck_path => q[file.fastqcheck], schema => $s); }
    qr/Fastqcheck file for file\.fastqcheck not found/,  'error when file is not found anywhere'; 
}

{  
  local $ENV{dev} = q[test];
  my $s = npg_qc::Schema->connect();
  $s->deploy;
  
  my $text = slurp q[t/data/2549_1_2.fastqcheck];
  $s->resultset('Fastqcheck')->update_or_create(
             {file_name => q[2549_1_2.fastqcheck], file_content => $text,});

  my $fq = npg_common::fastqcheck->new(
        fastqcheck_path => q[wrong_path/2549_1_2.fastqcheck], schema => $s);

  is($fq->num_reads, 9_855_354, q{number of sequences is correct});
  is($fq->total_pf_bases(), 532_189_116, q{total_pf_bases is correct});
  is($fq->average_read_length(), '54.00', q{average cycle count is correct});
  is($fq->max_read_length(), 54, q{max cycle count is correct});
}

1;