use strict;
use warnings;
use Carp;
use English qw{-no_match_vars};
use Test::More tests => 19;
use Test::Exception;
use lib qw{t t/lib};
use File::Temp qw(tempdir);
use Cwd;

BEGIN {
  use_ok(q{npg_common::role_tests::run_long_info});
}


my $basedir = tempdir( CLEANUP => 1 );
$ENV{TEST_DIR} = $basedir; #so when npg_common::roles::run::path_info globs the test directory

my $id_run = q{1234};
my $name = q{IL2_1234};
my $run_folder = q{123456_IL2_1234};
my $runfolder_path = qq{$basedir/staging/IL2/analysis/123456_IL2_1234};
my $data_subpath = $runfolder_path . q{/Data};
my $intensities_subpath = $data_subpath . q{/Intensities};
my $basecalls_subpath = $intensities_subpath . q{/BaseCalls};
my $bustard_subpath = $intensities_subpath . q{/Bustard-2009-10-01};
my $gerald_subpath = $bustard_subpath . q{/GERALD-2009-10-01};
my $archive_subpath = $gerald_subpath . q{/archive};
my $qc_subpath = $archive_subpath . q{/qc};
my $config_path = $runfolder_path . q{/Config};

sub delete_staging {
  `rm -rf $basedir/staging`;
  return 1;
}

sub create_staging {
  delete_staging();
  `mkdir -p $qc_subpath`;
  `mkdir $basecalls_subpath`;
  `mkdir $config_path`;
  `cp t/data/recipes/Recipe_GA2-PEM_MP_2x76Cycle+8_v7.7.xml $runfolder_path/`;
  `cp t/data/recipes/TileLayout.xml $config_path/`;
  return 1;
}

sub create_latest_summary_link {
  create_staging();
  `ln -s $gerald_subpath $runfolder_path/Latest_Summary`;
  return 1;
}

my $orig_dir = getcwd();

{
  my $long_info;
  lives_ok  { $long_info = npg_common::role_tests::run_long_info->new({id_run => 1234}); } q{created role_test object ok};

  create_staging();

  is($long_info->use_bases(), q{Y76,I7n,y76}, q{use_bases string is correct});
  is($long_info->read_config_string(), q{Y76,I7n,y76}, q{read_config_string returns same as use_bases});

  is($long_info->is_paired_read(), 1, q{Read is paired});
  is($long_info->is_indexed(), 1, q{Read is indexed});

  is($long_info->lane_count(), 8, q{correct number of lanes});

  is($long_info->expected_cycle_count(), 160, q{correct number of expected cycles});
  is($long_info->cycle_count(), 160, q{cycle count returns the same as expected_cycle_count});

  is($long_info->tilelayout_columns(), 2, q{correct number of tilelayout_columns});
  is($long_info->tilelayout_rows(), 60, q{correct number of tilelayout_rows});
  is($long_info->tile_count(), 120, q{correct number of tiles});

  is($long_info->is_rta(), 1, q{run is rta});
}

chdir $orig_dir; #need to leave directories before you can delete them....
eval { delete_staging(); } or do { carp 'unable to delete staging area'; };

#Now let's test a new HiSeq directory....
$ENV{TEST_DIR} = 't/data/';
{
  my $long_info;
  lives_ok  { $long_info = npg_common::role_tests::run_long_info->new({id_run => 5281}); } q{created role_test object ok};
  cmp_ok($long_info->tile_count, '==', 32, 'correct tile count');
  cmp_ok($long_info->lane_count, '==', 8, 'correct lane count');
  cmp_ok($long_info->cycle_count, '==', 200, 'correct cycle count');
  cmp_ok($long_info->use_bases, 'eq', 'Y100,y100', 'correct use_bases string');
note($long_info->runfolder_path);

  my $test_lane_tile_clustercount = {};
  foreach my $i (1..8) {
    foreach my $y (1..8) {
      foreach my $add ( 0,20,40,60 ) {
        $test_lane_tile_clustercount->{$i}->{$y + $add} = undef;
      }
    }
  }

  is_deeply( $long_info->lane_tile_clustercount(), $test_lane_tile_clustercount, q{lane_tile_clustercount as expected} );
}

1;
